@@ -13,7 +13,10 @@ use namespace::clean;
 
 =head1 NAME
 
-Data::Transpose - Transpose, iterate, filter and validate data
+Data::Transpose - iterate, filter and validate data, and transpose to
+different field names
+
+=head1 DESCRIPTION
 
 Caters to your needs for manipulating data by different operations,
 which are filtering records, iterating records, validating and
@@ -21,17 +24,47 @@ transposing to different field names.
 
 =head1 VERSION
 
-Version 0.0013
+Version 0.0014
 
 =cut
 
-our $VERSION = '0.0013';
+our $VERSION = '0.0014';
 
 =head1 SYNOPSIS
 
-    use Data::Transpose;
+    use warnings;
+    use strict;
+    
+    use Data::Transpose::Prefix;
+    use Data::Dumper;
+    
+    my $data = {
+        first => 'John',
+        last  => 'Doe',
+        foo   => 'bar',
+    };
+    
+    my $dtp = Data::Transpose::Prefix->new(prefix => 'user.');
+    foreach my $needs_prefix ( qw(first last) ) {
+        $dtp->field( $needs_prefix );
+    }
+    
+    my $output = $dtp->transpose( $data );
+    
+    print Data::Dumper->Dump([$data, $output], [qw(data output)]);
 
-    my $tp = Data::Transpose->new;
+outputs:
+
+    $data = {
+              'first' => 'John',
+              'last' => 'Doe',
+              'foo' => 'bar'
+            };
+    $output = {
+                'user.last' => 'Doe',
+                'user.first' => 'John',
+                'foo' => 'bar'
+              };
 
 =head1 REFERENCE
 
@@ -163,7 +196,7 @@ sub transpose {
             $weed_value = $fld->value($vref->{$fld_name});
         }
         else {
-            $weed_value = $fld->value;
+            $weed_value = $fld->value(undef);
         }
 
         if ($new_name = $fld->target) {
@@ -266,7 +299,8 @@ L<http://search.cpan.org/dist/Data-Transpose/>
 
 =head1 ACKNOWLEDGEMENTS
 
-Todd Wade for mxcheck test failures fix (GH #5, #11).
+Todd Wade for mxcheck test failures fix (GH #5, #11)
+and improvements to documentation (GH #12, #13).
 
 =head1 LICENSE AND COPYRIGHT
 
